/* 
 * Copyright (C) 2007 Igor Popik <thrull at bezsensu.pl>
 * http://www.thrull.com
 */

// reference to the interface defined in nsICookieAutoCompleteSearch.idl
const nsICookieAutoCompleteSearch = Components.interfaces.nsICookieAutoCompleteSearch;

// reference to the required base interface that all components must support
const nsIAutoCompleteSearch = Components.interfaces.nsIAutoCompleteSearch;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{1e27a0e4-c779-11db-8314-0800200c9a66}");

// description
const CLASS_NAME = "CookieAutoCompleteSearch Javascript XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=cookie-watcher-autocomplete";

//class constructor
function CookieAutoCompleteSearch() {
	// good practice says that we should use Object not an Array
	this.cookies = new Object();
};

// class definition
CookieAutoCompleteSearch.prototype = {

  // startSearch
  startSearch: function(searchString, searchParam, previousResult, listener) {
	var resultElements = [];
	var reg = new RegExp("^" + searchString, "i");

	for (i in this.cookies) {
		if (reg.test(i)) {
			resultElements.push(i);
		}
	}

	var result = new AutoCompleteResult(searchString, resultElements);
	listener.onSearchResult(this, result);
  },
  // this function adds elements to searchable list
  addCookieToList: function(cookieName) {
      this.cookies[cookieName] = cookieName;
  },

  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsICookieAutoCompleteSearch) &&    
        !aIID.equals(AutoCompleteSearch))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


/*
 * AutoCompleteResult - class implements nsIAutoCompleteResult
 */

// class constructor
function AutoCompleteResult(search, elements) {
 	        this.search = search;
 	        this.cookies = elements;
}
//class prototype
AutoCompleteResult.prototype = {
 	        get defaultIndex() {
 	                return 0;
 	        },
 	        get errorDescription() {
 	                return '';
 	        },
 	        get matchCount() {
 	                return this.cookies.length;
 	        },
 	        get searchResult() {
 	                return Components.interfaces.nsIAutoCompleteResult.RESULT_SUCCESS;
 	        },
 	        get searchString() {
 	                return this.search;
 	        },
 	        getCommentAt: function(index) {
 	                return '';
 	        },
 	        getStyleAt: function(index) {
 	                return '';
 	        },
 	        getValueAt: function(index) {
 	                return this.cookies[index];
 	        },
 	        removeValueAt: function(rowIndex, removeFromDb) {
 	        },
	 	QueryInterface: function (uuid) {
 	                if (uuid.equals(Components.interfaces.nsIAutoCompleteResult) ||
 	                        uuid.equals(Components.interfaces.nsISupports)) {
 	                        return this;
 	                }
 	        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
 	        return null;
 	    }
}


/*
 * class factory
 */

var CookieAutoCompleteSearchFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new CookieAutoCompleteSearch()).QueryInterface(aIID);
  }
};

/*
 * module definition (xpcom registration)
 */
var CookieAutoCompleteSearchModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return CookieAutoCompleteSearchFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/*
 * module initialization
 */

function NSGetModule(aCompMgr, aFileSpec) { return CookieAutoCompleteSearchModule; }

